import 'dart:collection';
import 'package:advertising_id/advertising_id.dart';

import 'package:app_tracking_transparency/app_tracking_transparency.dart';
import 'package:cramr_ui_lib/ads/preloaded_banner_ad.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';

class AdsController {
  final MobileAds _instance;

  bool _isPersonalisedAllowed = false;

  bool get isPersonalisedAllowed => _isPersonalisedAllowed;

  final Map<String, PreloadedBannerAd> _preloadedAd = HashMap();

  AdRequest androidAdRequest = const AdRequest();
  AdRequest personalisedAdRequest = const AdRequest(nonPersonalizedAds: false);
  AdRequest nonPersonalisedAdRequest =
      const AdRequest(nonPersonalizedAds: true);

  AdsController(this._instance);

  AdRequest createAdRequest() {
    return isPersonalisedAllowed
        ? personalisedAdRequest
        : nonPersonalisedAdRequest;
  }

  void dispose(String adUnitId) {
    _preloadedAd[adUnitId]?.dispose();
    var removed = _preloadedAd.remove(adUnitId);
  }

  Future<void> initialize() async {
    _instance.initialize();
    Future<TrackingStatus> status = isTrackingAllowed();
    status.then((value) {
      _isPersonalisedAllowed = (TrackingStatus.authorized == value);
    });
    String? advertisingId = await AdvertisingId.id(true);
  }

  Future<TrackingStatus> isTrackingAllowed() async {
    final status = await AppTrackingTransparency.requestTrackingAuthorization();
    return status;
  }

  /// Starts preloading an ad to be used later.
  ///
  /// The work doesn't start immediately so that calling this doesn't have
  /// adverse effects (jank) during start of a new screen.
  void preloadAd(String adUnitId) {
    // Wait a bit so that calling at start of a new screen doesn't have
    // adverse effects on performance.
    Future<void>.delayed(const Duration(seconds: 1)).then((_) {
      _preloadedAd[adUnitId] =
          PreloadedBannerAd(size: AdSize.banner, adUnitId: adUnitId);
      return _preloadedAd[adUnitId]?.load();
    });
  }

  /// Allows caller to take ownership of a [PreloadedBannerAd].
  ///
  /// If this method returns a non-null value, then the caller is responsible
  /// for disposing of the loaded ad.
  PreloadedBannerAd? takePreloadedAd(String adUnitId) {
    final ad = _preloadedAd[adUnitId];
    if (ad != null) {
      ad.dispose();
      var removed = _preloadedAd.remove(adUnitId);
    }
    return ad;
  }
}
