import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';

class SafeBannerAdWidget extends StatefulWidget {
  const SafeBannerAdWidget(
      {Key? key, required this.adUnitId, required this.adRequest})
      : super(key: key);

  @override
  State<SafeBannerAdWidget> createState() => SafeBannerAdWidgetState();

  final String adUnitId;
  final AdRequest adRequest;
}

class SafeBannerAdWidgetState extends State<SafeBannerAdWidget> {
  AdLoadingState loadingState = AdLoadingState.loading;
  late BannerAd ad;

  @override
  void initState() {
    super.initState();
    createBannerAd();
  }

  void createBannerAd() {
    ad = BannerAd(
        size: AdSize.banner,
        adUnitId: widget.adUnitId,
        listener: listener(),
        request: widget.adRequest);
    ad.load();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height * 0.1,
        child: loadingState == AdLoadingState.loaded
            ? AdWidget(ad: ad)
            : const Text(''));
  }

  BannerAdListener listener() {
    return BannerAdListener(onAdLoaded: (Ad ad) {
      setState(() {
        loadingState = AdLoadingState.loaded;
        if (kDebugMode) {
          print('called setState in AdLoaded');
        }
      });
    }, onAdFailedToLoad: (Ad ad, AdError error) {
      setState(() {
        loadingState = AdLoadingState.failedToLoad;
        if (kDebugMode) {
          print('called setState in AdFailedToLoad for adUnitId');
          print(widget.adUnitId);
          print(error.message);
        }
      });
    });
  }

  @override
  void dispose() {
    ad.dispose();
    super.dispose();
  }
}

enum AdLoadingState { loading, disposed, loaded, failedToLoad }
