import 'dart:collection';
import 'package:cramr_ui_lib/model/practice_session.dart';
import 'package:flutter/material.dart';

// TODO: make singleton
class StreakEngine with ChangeNotifier {
  Queue<PracticeSession> _sessions = Queue();

  Queue<PracticeSession> get sessions => _sessions;

  StreakEngine() {
    _sessions = Queue();
  }

  bool hasSessions() {
    return _sessions.isNotEmpty;
  }

  int numberOfSessions() {
    return _sessions.length;
  }

  void addSession(PracticeSession session) {
    _sessions.addLast(session);
    notifyListeners();
  }

  PracticeSession lastSession() {
    return _sessions.last;
  }

  bool hasValidStreak() {
    if (hasSessions() && _sessions.length > 1) {
      for (int i = _sessions.length - 1; i > 1; i--) {
        int j = i - 1;
        DateKey top = _sessions.elementAt(i).dateKey();
        DateKey bottom = _sessions.elementAt(j).dateKey();
        // test for same date which is ok
        if (top == bottom) {
          continue;
        } else if (bottom.isDayBefore(top)) {
          return true;
        } else {
          return false;
        }
      }
    }
    return false;
  }

  int streakCount() {
    int streakCount = 0;
    if (hasSessions()) {
      if (_sessions.length == 1 && _sessions.elementAt(0).dateKey().isToday()) {
        return 1;
      } else if (_sessions.length > 1) {
        DateKey current = _sessions.elementAt(_sessions.length - 1).dateKey();
        streakCount++;
        for (int i = _sessions.length - 2; i >= 0; i--) {
          DateKey comparable = _sessions.elementAt(i).dateKey();
          // test for same date which is ok
          if (current == comparable) {
            continue;
          } else if (comparable.isDayBefore(current)) {
            streakCount++;
            current = comparable;
            continue;
          } else {
            break;
          }
        }
      }
    }
    return streakCount;
  }

  List<PracticeSession> sessionsForDateKey(DateKey arg) {
    List<PracticeSession> result = [];
    for (var element in _sessions) {
      if (element.dateKey() == arg) {
        result.add(element);
      }
    }
    return result;
  }

  Map<String, dynamic> toJson() => {
        'sessions':
            _sessions.map((session) => PracticeSession.toJson(session)).toList()
      };

  StreakEngine.fromJson(Map<String, dynamic> json) {
    Iterable<dynamic> iterable = json['sessions'];
    for (var element in iterable) {
      Map<String, dynamic> el = element as Map<String, dynamic>;
      PracticeSession session = PracticeSession.fromJson(el);
      _sessions.add(session);
    }
  }
}
