import 'package:app_tracking_transparency/app_tracking_transparency.dart';
import 'package:cramr_ui_lib/ads/ad_state.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:flutter/material.dart';

class ConjugationSelectionFooterAdWidget extends StatefulWidget {
  ConjugationSelectionFooterAdWidget({Key? key}) : super(key: key);

  late bool _isBottomAdImpressed = false;

  bool get isBottomAdImpressed => _isBottomAdImpressed;

  @override
  State<ConjugationSelectionFooterAdWidget> createState() {
    return ConjugationSelectionFooterAdWidgetState();
  }

  static ConjugationSelectionFooterAdWidgetState? getState(
      BuildContext context) {
    return context
        .findAncestorStateOfType<ConjugationSelectionFooterAdWidgetState>();
  }

  static bool isBottomBannerAdLoaded(BuildContext context) {
    ConjugationSelectionFooterAdWidgetState? state = getState(context);
    if (state == null) {
      return false;
    } else {
      return state._isBottomBannerAdLoaded;
    }
  }
}

class ConjugationSelectionFooterAdWidgetState
    extends State<ConjugationSelectionFooterAdWidget> {
  late BannerAd _bottomBannerAd;
  bool _isPersonalisedAllowed = false;
  bool _isBottomBannerAdLoaded = false;

  static const AdRequest _nonPersonalisedAdRequest =
      AdRequest(nonPersonalizedAds: true);
  static const AdRequest _personalisedAdRequest =
      AdRequest(nonPersonalizedAds: false);

  @override
  Widget build(BuildContext context) {
    return Container(
      width: _bottomBannerAd.size.width.toDouble(),
      height: 100.0,
      alignment: Alignment.center,
      child: AdWidget(ad: _bottomBannerAd),
    );
  }

  @override
  void initState() {
    super.initState();
    Future<TrackingStatus> status = isTrackingAllowed();
    status.then((value) {
      _isPersonalisedAllowed = (TrackingStatus.authorized == value);
    });
    _createBottomBannerAd();
  }

  @override
  void dispose() {
    _bottomBannerAd.dispose();
    super.dispose();
  }

  Future<TrackingStatus> isTrackingAllowed() async {
    final status = await AppTrackingTransparency.requestTrackingAuthorization();
    return status;
  }

  void _createBottomBannerAd() {
    _bottomBannerAd = BannerAd(
      adUnitId: AdState.conjugationSelectionFooterAdUnitId,
      size: AdSize.banner,
      request: _isPersonalisedAllowed
          ? _personalisedAdRequest
          : _nonPersonalisedAdRequest,
      listener: BannerAdListener(
          onAdLoaded: (_) {},
          onAdFailedToLoad: (ad, error) {
            ad.dispose();
          },
          onAdImpression: (ad) {
            setState(() {
              _isBottomBannerAdLoaded = true;
              widget._isBottomAdImpressed = true;
            });
          }),
    );
    _bottomBannerAd.load();
  }
}
