import 'package:flutter_test/flutter_test.dart';
import 'package:cramr_ui_lib/model/practice_session.dart';

void main() {
  final DateTime thisDay = DateTime.now();
  final DateKey thisDayKey = DateKey(thisDay.year, thisDay.month, thisDay.day);
  const String mode = '_mode';
  const String tense = '_tense';
  final List<String> actuals = ['suis', 'es', 'est', 'sommes', 'êtes', 'sont'];

  group('some practice session tests', () {
    test('some practice session test', () {
      PracticeSession session = PracticeSession(
          thisDay, '_mode', '_tense', [''], StudyMethod.coverMethod);
      expect(session.dateKey(), thisDayKey);
    });
    test('toJson and fromJson test', () {
      PracticeSession original = PracticeSession(
          thisDay, mode, tense, actuals, StudyMethod.coverMethod);
      Map<String, dynamic> jsonMap = PracticeSession.toJson(original);
      PracticeSession parsed = PracticeSession.fromJson(jsonMap);
      expect(original.actuals, parsed.actuals);
      expect(original.dateKey(), parsed.dateKey());
      expect(original.mode, parsed.mode);
      expect(original.studyMethod, parsed.studyMethod);
      expect(original.tense, parsed.tense);
    });
  });
  group('some DateKey tests', () {
    test('test day before method', () {
      DateTime today = DateTime.now();
      DateKey todayKey = DateKey(today.year, today.month, today.day);
      Duration oneDay = const Duration(days: 1);
      DateTime yesterday = today.subtract(oneDay);
      DateKey yesterdayKey =
          DateKey(yesterday.year, yesterday.month, yesterday.day);
      expect(yesterdayKey.isDayBefore(todayKey), true);
      expect(todayKey.isDayBefore(todayKey), false);
    });
    test('test equals method', () {
      DateTime today = DateTime.now();
      DateKey todayKey = DateKey(today.year, today.month, today.day);
      DateTime stillToday = DateTime.now();
      DateKey stillTodayKey =
          DateKey(stillToday.year, stillToday.month, stillToday.day);
      Duration oneDay = const Duration(days: 1);
      DateTime yesterday = today.subtract(oneDay);
      DateKey yesterdayKey =
          DateKey(yesterday.year, yesterday.month, yesterday.day);
      expect(todayKey, stillTodayKey);
      expect(todayKey, isNot(equals(yesterdayKey)));
    });
    test('test isToday method', () {
      DateTime today = DateTime.now();
      DateKey todayKey = DateKey(today.year, today.month, today.day);
      Duration oneDay = const Duration(days: 1);
      DateTime yesterday = today.subtract(oneDay);
      DateKey yesterdayKey =
          DateKey(yesterday.year, yesterday.month, yesterday.day);
      expect(todayKey.isToday(), true);
      expect(yesterdayKey.isToday(), false);
    });
  });
}
