import 'package:cramr_ui_lib/model/conjugation.dart';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  var infinitive = 'infinitief';
  var translation = 'translation';
  var simplePresentIndicative = ['nul', 'een', 'twee', 'drie', 'vier', 'vijf'];
  var simplePastIndicative = [
    'oudnul',
    'oudeen',
    'oudtwee',
    'ouddrie',
    'oudvier',
    'oudvijf'
  ];
  var presentConditional = [
    'condnul',
    'condeen',
    'condtwee',
    'conddrie',
    'condvier',
    'condvijf'
  ];
  var simpleFutureIndicative = [
    'toekomstnul',
    'toekomsteen',
    'toekomsttwee',
    'toekomstdrie',
    'toekomstvier',
    'toekomstvijf',
  ];
  var isoLanguage = 'fr';

  group('Verb tests simple present indicative', () {
    test('Verb state tests', () {
      Conjugation conjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      var model = Verb(infinitive, translation, conjugation, isoLanguage);
      expect(infinitive, model.infinitive);
      expect(translation, model.translation);
      expect(isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in simplePresentIndicative) {
        expect(element, model.simplePresentIndicative[i++]);
      }
      if (null != model.simplePastIndicative) {
        fail('non null where null expected');
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation conjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      var model = Verb(infinitive, translation, conjugation, isoLanguage);
      expect(false, model.hasSimplePastIndicative());
    });
  });

  group('Verb tests several indicatives', () {
    test('Verb state tests two tenses', () {
      Conjugation presentConjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      Conjugation pastConjugation =
          Conjugation('indicatif', 'imparfait', simplePastIndicative);
      var model = Verb.fromMoreTensesAndModes(infinitive, translation,
          presentConjugation, pastConjugation, isoLanguage);
      expect(infinitive, model.infinitive);
      expect(translation, model.translation);
      expect(isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in simplePastIndicative) {
        expect(element, model.simplePastIndicative![i++]);
      }
    });

    test('Verb state tests three tenses', () {
      Conjugation presentConjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      Conjugation pastConjugation =
          Conjugation('indicatif', 'imparfait', simplePastIndicative);
      Conjugation futureConjugation =
          Conjugation('indicatif', 'futur simple', simpleFutureIndicative);
      var model = Verb.fromAllTensesAndModes(
          infinitive,
          translation,
          [presentConjugation, pastConjugation, futureConjugation],
          isoLanguage);
      expect(infinitive, model.infinitive);
      expect(translation, model.translation);
      expect(isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in simplePastIndicative) {
        expect(element, model.simplePastIndicative![i++]);
      }
      i = 0;
      for (var element in simpleFutureIndicative) {
        expect(element, model.simpleFutureIndicative[i++]);
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation presentConjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      Conjugation pastConjugation =
          Conjugation('indicatif', 'imparfait', simplePastIndicative);
      var model = Verb.fromMoreTensesAndModes(infinitive, translation,
          presentConjugation, pastConjugation, isoLanguage);
      expect(true, model.hasSimplePastIndicative());
    });

    test('Verb tests mixed modes and tenses with duplicates', () {
      Conjugation presentConjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      Conjugation pastConjugation =
          Conjugation('conditionnel', 'présent', presentConditional);
      Conjugation futureConjugation =
          Conjugation('indicatif', 'futur simple', simpleFutureIndicative);
      var model = Verb.fromAllTensesAndModes(
          infinitive,
          translation,
          [presentConjugation, pastConjugation, futureConjugation],
          isoLanguage);
      expect(infinitive, model.infinitive);
      expect(translation, model.translation);
      expect(isoLanguage, model.isoLanguage);
      expect(3, model.indicativeConjugations.length);
    });

    test('test the conjugation method', () {
      Conjugation presentConjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      Conjugation pastConjugation =
          Conjugation('indicatif', 'imparfait', simplePastIndicative);
      Conjugation futureConjugation =
          Conjugation('indicatif', 'futur simple', simpleFutureIndicative);
      var model = Verb.fromAllTensesAndModes(
          infinitive,
          translation,
          [presentConjugation, pastConjugation, futureConjugation],
          isoLanguage);
      expect(infinitive, model.infinitive);
      expect(translation, model.translation);
      expect(isoLanguage, model.isoLanguage);
      expect(model.conjugation('indicatif', 'imparfait')?.tense, 'imparfait');
      expect(model.conjugation('indicatif', 'imparfait')?.mode, 'indicatif');
      expect(model.conjugation('conditionnel', 'imparfait'), isNull);
    });
  });
}
