import 'package:test/test.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';

void main() {
  group('Verb Test Model tests', () {
    var expectedFull = ["ai", "as", "a", "avons", "avez", "ont"];
    var expectedSingular = expectedFull.sublist(0, 2);
    var expectedPlural = expectedFull.sublist(3, 5);
    var pronouns = ["je", "tu", "il", "nous", "vous", "ils"];
    var pronounsSingular = pronouns.sublist(0, 2);
    var pronounsPlural = pronouns.sublist(3, 5);
    var targetLanguageIso = "fr";
    var infinitive = "avoir";
    var translation = "hebben";
    var mode = 'indicatif';
    var tense = 'présent';

    test('Some random behaviour tests for the model', () {
      var model = VerbTestModel();
      model.initialise(expectedFull, pronouns, infinitive,
          targetLanguageIso, translation, mode, tense);
      model.add("banana", 3);

      expect(model.isComplete(), false);
      expect(model.score, 0);
      expect(model.items.length, 6);
      expect(model.items[0], "");
      expect(model.items[3], "banana");
      expect(model.pronouns.length, pronouns.length);
      expect(model.pronouns[1], pronouns[1]);
      expect(model.translation, translation);
      expect(model.tense, tense);
      expect(model.mode, mode);

      model.reset();

      expect(model.items.length, 6);
      expect(model.items[3], "");
      expect(model.translation, translation);
    });

    test('Test the trimming of the leading spaces', () {
      var model = VerbTestModel();
      model.initialise(expectedFull, pronouns, infinitive,
          targetLanguageIso, translation, mode, tense);
      model.reset();
      model.add(" leading", 0);
      expect(model.items[0], "leading");
    });

    test('Test the trimming of the trailing spaces', () {
      var model = VerbTestModel();
      model.initialise(expectedFull, pronouns, infinitive,
          targetLanguageIso, translation, mode, tense);
      model.reset();
      model.add("trailing ", 0);
      expect(model.items[0], "trailing");
    });

    test('test isComplete method, focus on leading and trailing spaces', () {
      var model = VerbTestModel();
      model.initialise(expectedFull, pronouns, infinitive,
          targetLanguageIso, translation, mode, tense);
      model.reset();
      model.add(" leading", 0);
      expect(model.isComplete(), false);
      model.add("trailing ", 1);
      expect(model.isComplete(), false);
      model.add(" ", 2);
      expect(model.isComplete(), false);
      model.add(" ", 3);
      expect(model.isComplete(), false);
      model.add(" ", 4);
      expect(model.isComplete(), false);
      model.add(" ", 5);
      expect(model.isComplete(), false);

      model.add("something", 3);
      expect(model.isComplete(), false);

      model.add(" haha", 4);
      expect(model.isComplete(), false);

      model.add(" haha", 5);
      expect(model.isComplete(), false);

      model.add("dont forget", 2);
      expect(model.isComplete(), true);
    });

    test('test singular only', () {
      var model = VerbTestModel();
      model.initialise(expectedSingular, pronounsSingular, infinitive,
          targetLanguageIso, translation, mode, tense);
      expect(model.isComplete(), false);
      expect(model.pronouns.length, pronounsSingular.length);
      var pronounCounter = 0;
      for (var element in pronounsSingular) {
        expect(model.pronouns[pronounCounter++], element);
      }
      expect(0, model.score);
      var singularCounter = 0;
      for (var element in expectedSingular) {
        model.add(element, singularCounter++);
        expect(model.score, singularCounter);
      }
      expect(model.isComplete(), true);
    });

    test('test plural only', () {
      var model = VerbTestModel();
      model.initialise(expectedPlural, pronounsPlural, infinitive,
          targetLanguageIso, translation, mode, tense);
      expect(model.isComplete(), false);
      expect(model.pronouns.length, pronounsPlural.length);
      var pronounCounter = 0;
      for (var element in pronounsPlural) {
        expect(model.pronouns[pronounCounter++], element);
      }
      expect(0, model.score);
      var pluralCounter = 0;
      for (var element in expectedPlural) {
        model.add(element, pluralCounter++);
        expect(model.score, pluralCounter);
      }
      expect(model.isComplete(), true);
    });
  });
}
