import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_flashcard_mode_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

final List<String> _randomIndicative = [
  'aime',
  'aimes',
  'aime',
  'aimons',
  'aimez',
  'aiment',
];

final List<String> _longIndicative = [
  'viens de comprendre',
  'viens de comprendre',
  'vient de comprendre',
  'venons de comprendre',
  'venons de comprendre',
  'viennent de comprendre'
];

final List<String> _randomPronouns = [
  'je',
  'tu',
  'il/elle',
  'nous',
  'vous',
  'ils/elles',
];

const String _randomInfinitive = 'infinitive';
final Finder sliderFinder = find.byKey(const Key('flashcard_slider'));
final Finder playButtonFinder = find.byKey(const Key('PlayButton'));
final Finder resetButtonFinder = find.byKey(const Key('ResetButton'));

const String _mode = 'mode';
const String _tense = 'tense';

void main() {
  testWidgets('Test the flashcard mode study screen for all 6 pronouns',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(_randomIndicative, _randomPronouns,
                _randomInfinitive, 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.flashCardMode);
          })
        ],
        child: VerbIndicativeStudyFlashcardModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    Finder findFirstIndicative = find.text(_randomIndicative[0]);
    expect(findFirstIndicative, findsWidgets);
    expect(sliderFinder, findsOneWidget);
    expect(playButtonFinder, findsOneWidget);
    expect(resetButtonFinder, findsNothing);
    await tester.tapAt(Offset(tester.getCenter(sliderFinder).dx - 70,
        tester.getCenter(sliderFinder).dy));
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 3));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
  });

  testWidgets('Test the flashcard mode study screen for singular only',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(true, false);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicativeList, filteredPronounList,
                _randomInfinitive, 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.flashCardMode);
          })
        ],
        child: VerbIndicativeStudyFlashcardModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    Finder findFirstIndicative = find.text(filteredIndicativeList[0]);
    expect(findFirstIndicative, findsWidgets);
    expect(sliderFinder, findsOneWidget);
    expect(playButtonFinder, findsOneWidget);
    expect(resetButtonFinder, findsNothing);
    await tester.tapAt(Offset(tester.getCenter(sliderFinder).dx - 70,
        tester.getCenter(sliderFinder).dy));
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 3));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
  });

  testWidgets('Test the flashcard mode study screen for plural only',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(false, true);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicativeList, filteredPronounList,
                _randomInfinitive, 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.flashCardMode);
          })
        ],
        child: VerbIndicativeStudyFlashcardModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    Finder findFirstIndicative = find.text(filteredIndicativeList[0]);
    expect(findFirstIndicative, findsWidgets);
    expect(sliderFinder, findsOneWidget);
    expect(playButtonFinder, findsOneWidget);
    expect(resetButtonFinder, findsNothing);
    await tester.tapAt(Offset(tester.getCenter(sliderFinder).dx - 70,
        tester.getCenter(sliderFinder).dy));
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 3));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
  });

  testWidgets('Test the flashcard mode study screen for long conjugations',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(_longIndicative, _randomPronouns,
                _randomInfinitive, 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.flashCardMode);
          })
        ],
        child: VerbIndicativeStudyFlashcardModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    Finder findFirstPronoun = find.text(_randomPronouns[0]);
    expect(findFirstPronoun, findsOneWidget);
    _longIndicative[0].split(' ').forEach((element) {
      Finder findIndicativePart = find.text(element);
      expect(findIndicativePart, findsWidgets);
    });
    expect(sliderFinder, findsOneWidget);
    expect(playButtonFinder, findsOneWidget);
    expect(resetButtonFinder, findsNothing);
    await tester.tapAt(Offset(tester.getCenter(sliderFinder).dx - 70,
        tester.getCenter(sliderFinder).dy));
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 3));
    expect(resetButtonFinder, findsOneWidget);
    await tester.tap(resetButtonFinder);
    await tester.pumpAndSettle();
    expect(playButtonFinder, findsOneWidget);
    await tester.tap(playButtonFinder);
    await tester.pumpAndSettle(const Duration(seconds: 6));
  });
}
